/*

f2c - UNIX utility to convert files into C source bitmap.

Copyright (C) 2006  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#define USAGE_MSG "USAGE: bin2head <input_file> [<output_file>]\n"
#define INFO "/*\n\bBinary image of: <%s>\n*/\n\n"

int main(int argc, char *argv[]){
   unsigned char byte;
   int fdIn=-1, fdOut=-1, i, r, exit_status=0, count, size;
   char nameOut[256], line[256];

   if(argc<2)
      goto uerror;

   if((fdIn = open(argv[1],O_RDONLY))<0)
      goto ferror;

   if(argc<3){
      char *dot;
      sprintf(nameOut,"%s",argv[1]);

      dot = nameOut + strlen(nameOut);
      *dot = '.';
      *(++dot) = 'c';
      *(++dot) = 0;
   }
   else{
      sprintf(nameOut,"%s",argv[2]);
   }

   if((fdOut = open(nameOut, O_WRONLY | O_CREAT | O_TRUNC, S_IRWXU))<0)
      goto ferror;

   strcpy(nameOut,argv[1]);

   for(i=0;i<strlen(nameOut);i++)
      if(nameOut[i]=='.') nameOut[i]='_';

   count = sprintf(line,INFO"unsigned char %s[]={\n",argv[1],nameOut,nameOut,nameOut);
   if(write(fdOut,line,count)<0)
      goto werror;

   for(size=i=1;(r=read(fdIn,&byte,sizeof(unsigned char)))>0;size++,i=(i+1)%8){
      count = sprintf(line,"\t0x%X,%c",byte,(i)? 32 :'\n');
      if(write(fdOut,line,count)<0)
         goto werror;
   }

   count = sprintf(line,"\t0x0\n};\n\n/* %s size = %d; */\n\n",nameOut,size);
   if((write(fdOut,line,count))<0)
      goto werror;

   goto finalize;

ferror:
   printf("Error occurred while opening file: %s\n",(fdIn<0) ? argv[1] : nameOut);
   exit_status = -1;
   goto finalize;
uerror:
   printf("%s",USAGE_MSG);
   exit_status = 0;
   goto finalize;
werror:
   printf("Error while writing output file\n");
   exit_status = -1;
   goto finalize;
finalize:
   if(fdIn>=0)
      close(fdIn);
   if(fdOut>=0)
      close(fdOut);
   return (exit_status);
}
