/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef _GDT_H_
#define _GDT_H_

#include<types.h>

#define GDT_SIZE 256

struct GDTgate_t{
   word LimitL;
   word BaseL;
   byte BaseHL;
      struct{
         dword Type:4;
         dword S:1;
         dword DPL:2;
         dword Present:1;
         dword LimitH:4;
         dword AVL:1;
         dword L:1;
         dword DB:1;
         dword Granularity:1;
      }__attribute__((packed)) Options;
   byte BaseHH;
}__attribute__((packed));


#define LDTgate_t GDTgate_t

/* GDT entry TYPE FIELD, S flag is included */

/* S=1 */
/* READ ONLY */
#define GDT_DATA_RO  0x10
/* READ ONLY, ACCESSED */
#define GDT_DATA_RO_A 0x11
/* READ WRITE */
#define GDT_DATA_RW 0x12
/* READ WRITE, ACCESSED */
#define GDT_DATA_RW_A 0x13
/* READ ONLY, EXPAND DOWN */
#define GDT_DATA_RO_ED 0x14
/* READ ONLY, EXPAND DOWN, ACCESSED */
#define GDT_DATA_RO_ED_A 0x15
/* READ WRITE, EXPAND DOWN */
#define GDT_DATA_RW_ED 0x16
/* READ WRITE, EXPAND DOWN, ACCESSED */
#define GDT_DATA_RW_ED_A 0x17
/* EXECUTE ONLY */
#define GDT_CODE_EO 0x18
/* EXECUTE ONLY, ACCESSED */
#define GDT_CODE_EO_A 0x19
/* EXECUTE READ */
#define GDT_CODE_ER 0x1A
/* EXECUTE READ, ACCESSED */
#define GDT_CODE_ER_A 0x1B
/* EXECUTE ONLY, CONFORMING */
#define GDT_CODE_EO_C 0x1C
/* EXECUTE ONLY CONFORMING, ACCESSED */
#define GDT_CODE_EO_C_A 0x1D
/* EXECUTE READ ONLY, CONFORMING */
#define GDT_CODE_ERO_C 0x1E
/* EXECUTE READ ONLY, CONFORMING ACCESSED */
#define GDT_CODE_ERO_C_A 0x1F

/* GDT entry TYPES */
/* S=0 */
#define GDT_LDT 0x2
#define GDT_TSS 0x9

#define GDT_DPL0 0x0
#define GDT_DPL3 0x3

#define MAX_SEG_SIZE 0xFFFFFFFF

#define GDT_USED 1

#define GDT_G_4KB 0x1
#define GDT_G_1B 0x0

#define SEG_SEL(IDX,TI,RPL) ((IDX<<3)+(TI<<2)+RPL)
#define TI_LDT 1
#define TI_GDT 0

#define GLOBAL_LDT 3

void initGDT(void);
struct GDTgate_t makeGDTgate(dword limit, dword base, byte G, byte type, byte DPL);
struct GDTgate_t makeNullGDTgate(void);
int setGDTgate(int idx, struct GDTgate_t gate);
int getFreeGDTgate(void);
void freeGDTgate(int i);

#endif
