/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef _IRQ_H_
#define _IRQ_H_

#include <io.h>
#include <idt.h>

/* enabling/disabling interrupt macros */
#define sti() __asm__ __volatile__("sti")
#define cli() __asm__ __volatile__("cli")

/* addresses of PICs */
#define M_PIC      0x20
#define M_PIC_MASK 0x21
#define S_PIC      0xA0
#define S_PIC_MASK 0xA1

#define TIMER_IRQNO 0
#define KEYB_IRQNO  1

/* init PIC 8259 */
void initIRQ(void);

/* enable an irq (1-15) */
void enableIRQ(int irqNum);

/* disable an irq (1-15) */
void disableIRQ(int irqNum);

/* set irq handler */
void setHandlerIRQ(int, void (*)(void));

/*
[PIC 8259 Master]
--------------------------------------
IRQ0 System Timer
IRQ1 Keyboard
IRQ2 Link to slave PIC
IRQ3 COM2/COM4
IRQ4 COM1/COM3
IRQ5 LPT2
IRQ6 FDC
IRQ7 LPT1

[PIC 8259 Slave]
--------------------------------------
IRQ8 Real Time Clock Chip
IRQ9 Networking adapter
IRQ10 Unused
IRQ11 Unused
IRQ12 Link to master PIC
IRQ13 Floating Processor Unit
IRQ14 HDC
IRQ15 Unused
*/

#endif
