/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <keyboard.h>

static volatile word keyb_status = 0x0;

/* ENTER scancode = 28*/
byte KeyMapIT[] = {
       0,   27,  '1',  '2',  '3',  '4',  '5',  '6',
     '7',  '8',  '9',  '0', '\'',    0,  127,    9,
     'q',  'w',  'e',  'r',  't',  'y',  'u',  'i',
     'o',  'p',    0,  '+', '\n',    0,  'a',  's',
     'd',  'f',  'g',  'h',  'j',  'k',  'l',    0,
       0, '\\',    0,    0,  'z',  'x',  'c',  'v',
     'b',  'n',  'm',  ',',  '.',  '-',    0,  '*',
       0,   32,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,  '-',    0,    0,    0,  '+',    0,
       0,    0,    0,    0,    0,    0,  '<',    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0
};

byte KeyMapIT_shift[] = {
       0,   27,  '!',  '"',    0,  '$',  '%',  '&',
     '/',  '(',  ')',  '=',  '?',  '^',  127,    9,
     'Q',  'W',  'E',  'R',  'T',  'Y',  'U',  'I',
     'O',  'P',    0,  '*',   13,    0,  'A',  'S',
     'D',  'F',  'G',  'H',  'J',  'K',  'L',    0,
       0,  '|',  '0',    0,  'Z',  'X',  'C',  'V',
     'B',  'N',  'M',  ';',  ':',  '_',    0,    0,
       0,   32,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,  '>',    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0 };

byte KeyMapIT_alt[] = {
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,  '[',  ']',   13,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,  '@',
     '#',    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0,    0,    0,    0,    0,    0,    0,    0,
       0 };

static void (*keybUserHandler)(byte) = NULL;

void (*keybSignal(void (*f)(byte)))(byte){
   void(*tmp)(byte)=*keybUserHandler;
   keybUserHandler=*f;
   return tmp;
}

void keyb_handler(void)
{
   byte scan;
   byte ascii=0;

   for(ascii=0;ascii<64;ascii++) port_wait(); /* wait a lot */

   scan = inport(0x60);
   ascii = inport(0x61);

   /* do beep beep */
   /*outport(0x61, ascii|0x80);
   outport(0x61, ascii^0x7F);*/
   outport(0x20, 0x20);


   switch(scan){
      case 42: /*LSHITF pressed*/
      case 54: /*RSHIFT pressed*/
         SET_BIT(keyb_status,SHIFT);
         break;
      case 56: /*ALT pressed*/
         SET_BIT(keyb_status,ALT);
         break;
      case 42+128: /*LSHITF released*/
      case 54+128: /*RSHIFT released*/
         CLEAR_BIT(keyb_status,SHIFT);
         break;
      case 56+128: /*ALT released*/
         CLEAR_BIT(keyb_status,ALT);
   }

   if(IS_KEYPRESS(scan)){

      if(scan<sizeof(KeyMapIT)/sizeof(KeyMapIT[0])){

      if(IS_SET_BIT(keyb_status,SHIFT))
         ascii = toupper(KeyMapIT[scan]);
      else if(IS_SET_BIT(keyb_status,ALT))
         ascii = KeyMapIT_alt[scan];
      else
         ascii = KeyMapIT[scan];

      if(keybUserHandler && ascii)
         keybUserHandler(ascii);
      }
   }
}

void initKeyboard()
{
   keybUserHandler = NULL;
   keyb_status = 0x0;
   setHandlerIRQ(KEYB_IRQNO,keyb_handler);
   enableIRQ(KEYB_IRQNO);
}
