/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <termio.h>

static volatile int ttio_x = 0;
static volatile int ttio_y = 0;
static queue_t buffer;

void ttio_keyboard_input(byte ascii);

static void ttio_command(char ascii);
static void ttio_putchar(char ascii);
static void ttio_canc(void);
static void ttio_newline(void);
static void ttio_tab(void);

void initTTIO(void){
   initVGA();
   initKeyboard();
   initQueue(&buffer);
   keybSignal(&ttio_keyboard_input);
   ttio_cls();
}

int ttio_write(char *str, int max){
   int i;
   for(i=0;str[i] && i<max;i++)
      ttio_command(str[i]);
   return i;
}

int ttio_read(char *buff, int size){
   int i;
   char c;
   while(tailOnQueue(buffer)!='\n');
   for(i=0;i<size;i++,buff++){
      c = dequeue(&buffer);
      if(c<0)
         return -1;
      else if(c!='\n')
         *buff = c;
      else
         break;
   }
   *buff = (char)0;
   return i;
}

void ttio_cls(void){
   ttio_x = ttio_y = 0;
   clearScreen();
}

static void ttio_move_next(void){
   if(ttio_x >= SCREEN_WIDTH-1){
      if(ttio_y >= SCREEN_HEIGHT-1){
         scrollUp();
         ttio_x = 0;
         ttio_y = SCREEN_HEIGHT - 1;
      }
      else{
         ttio_x = 0;
         ttio_y++;
      }
   }
   else{
      ttio_x++;
   }
   moveCursor(ttio_x, ttio_y);
}

static void ttio_move_prev(void){
   if(ttio_x > 0){
      ttio_x--;
   }
   else if(ttio_y > 0){
      ttio_x = SCREEN_WIDTH - 1;
      ttio_y--;
   }
   moveCursor(ttio_x, ttio_y);
}

static void ttio_command(char ascii){
   switch(ascii){
      case '\n': /* '\n' */
         ttio_newline();
         break;
      case '\t':
         ttio_tab();
      case 127: /* backspace */
      {
         ttio_canc();
         break;
      }
      default:
         ttio_putchar(ascii);
         break;
   }
}

/*
static void debug_queue(){
   char str[64];
   int i;
   sprintf(str,"{%d : %d}",countOnQueue(buffer),tailOnQueue(buffer));
   for(i=0;i<strlen(str);i++)
      putchar(i,0,str[i]);
}
*/

void ttio_keyboard_input(byte ascii){

   switch(ascii){
      case 0:
         break;
      case 127:
      {
         if(tailOnQueue(buffer)=='\n')
            break;

         if(enqueueUndo(&buffer)>=0)
            ttio_command(ascii);

         break;
      }
      default:
      {
         enqueue(&buffer, ascii);
         ttio_command(ascii);
      }
   }
}

static void ttio_newline(void){
/*   while(ttio_x<SCREEN_WIDTH-1)
      ttio_putchar(32);
   ttio_putchar(32);*/
   ttio_y++;
   ttio_x=0;
   if(ttio_y>=SCREEN_HEIGHT){
      ttio_x = SCREEN_WIDTH -1;
      ttio_y = SCREEN_HEIGHT -1;
      ttio_move_next();}
   else{
      moveCursor(ttio_x, ttio_y);}
}

static void ttio_putchar(char ch){
   char print = (ch>=32 && ch<127) ? ch : 32;
   putChar(ttio_x, ttio_y, print);
   ttio_move_next();
}

static void ttio_canc(void){
   ttio_move_prev();
   putChar(ttio_x, ttio_y, 32);
}

static void ttio_tab(void){
   int i;
   for(i=0;i<8;i++)
      ttio_putchar(' ');
}

#define MAX_LINE 1024
int kprintf(const char *fmt, ...){
   char buffer[MAX_LINE];
   va_list args;
   int i;

   va_start(args, fmt);
   i=vsprintf(buffer,fmt,args);
   va_end(args);

   if(i>=0)
      ttio_write(buffer, i);

   return i;
}

int kscanf(const char *fmt, ...){
   char buffer[MAX_LINE];
   va_list args;
   int i;

   if(ttio_read(buffer,MAX_LINE)>0){
      va_start(args, fmt);
      i=vsscanf(buffer,fmt,args);
      va_end(args);
      return i;
   }
   else
      return 0;
}

char *kgets(char *str){
   int count = ttio_read(str, 0xFF);
   return (count>0) ? str : NULL;
}

char *kputs(char *str){
   ttio_write(str, MAX_LINE);
   return str;
}
