/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <irq.h>

/* cache the PICs mask value*/
unsigned char m_pic_mask = 0xFB; /* dont mask IRQ2: this will disable slave PIC! */
unsigned char s_pic_mask = 0xFF;

void setHandlerIRQ(int irqNo, void (*hand)(void)){
   if(irqNo >=0 && irqNo <= 15){
      INTtable[0x20+irqNo]=hand; /*0x0...0x10 exceptions, 0x11...0x1F unused, 0x20...0x2F irq*/
      enableIRQ(irqNo);
   }
}

void initPic8259(void){
   
   /* OCW1 command: mask all IRQ */
   outport(M_PIC_MASK, 0xFF);
   outport(S_PIC_MASK, 0xFF);
   
   /* ICW1 command */
   outport_wait(M_PIC, 0x11);
   outport_wait(S_PIC, 0x11);
   
   /* ICW2 command */
   /* the map value must be multiple of 0x8 : 0x0, 0x10, 0x20, 0x30*/
   outport_wait(M_PIC_MASK, 0x20); /* map IRQ0 to 0x20, IRQ1 to 0x21 ecc. */
   outport_wait(S_PIC_MASK, 0x28); /* map IRQ9 to 0x28, IRQ10 to 0x29 ecc. */
   /* when IRQ1 is raised PIC returns 0x21 */
   
   /* ICW3 command */
   outport_wait(M_PIC_MASK, 0x04);
   outport_wait(S_PIC_MASK, 0x02);
   
   /* ICW4 command */
   outport_wait(M_PIC_MASK, 0x01);
   outport_wait(S_PIC_MASK, 0x01);
   
   /* get mask back to cached values */
   outport(M_PIC_MASK, m_pic_mask);
   outport(S_PIC_MASK, s_pic_mask);
}

void enableIRQ(int irqNum){
   if(irqNum < 0)
      return;
   else if(irqNum <= 7){ /* use master pic */
      /* turn irqNum(th) bit off (off means dont mask) */
      m_pic_mask &= ~(1<<irqNum); 
      outport(M_PIC_MASK, m_pic_mask);
   }
   else{ /* use slave pic */
      /* turn irqNum(th)-8 bit off (off means dont mask) */
      m_pic_mask &= ~(1<<(irqNum-8));
      outport(S_PIC_MASK, s_pic_mask);
   }
}

void disableIRQ(int irqNum){
   if(irqNum < 0)
      return;
   else if(irqNum <= 7){ /* use master pic */
      /* turn irqNum(th) bit on (on means mask) */
      m_pic_mask |= (1<<irqNum); 
      outport(M_PIC_MASK, m_pic_mask);
   }
   else{ /* use slave pic */
      /* turn irqNum(th)-8 bit on (on means mask) */
      m_pic_mask |= (1<<(irqNum-8));
      outport(S_PIC_MASK, s_pic_mask);
   }   
}

void initIRQ(void){
   initPic8259();
}
