/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <swapper.h>

extern char *memcpy(const char*, const char*, int);

static int count=0;
struct swap_map_t{
   int owner;
   dword address;
   char page[4096];
}*swap_map = (struct swap_map_t*)PHYS_MEM_LIMIT;

static int lookup(int owner, dword address){
   int i;
   for(i=0;i<count;i++){
      if(swap_map[i].owner == owner && swap_map[i].address == address)
         return i;
   }
   return -1;
}

int swap_out_shared(dword address, void *page){
   return swap_out(-1, address, page);
}

int swap_in_shared(dword address, void *buffer){
   return swap_in(-1, address, buffer);
}

int swap_out(int owner, dword address, void *page){
   dword info;
   int idx;

   info = BEGIN_PHYS_ADDR(0);

   address&=0xFFFFF000;

   if((idx=lookup(owner, address))<0){
      idx=count++;
   }
   memcpy(swap_map[idx].page, page, PAGE_SIZE);
   swap_map[idx].address = address;
   swap_map[idx].owner = owner;

   END_PHYS_ADDR(info,0);

  return 1;
}

int swap_in(int owner, dword address, void *buffer){
   dword info;
   int idx;
   info = BEGIN_PHYS_ADDR(0);

   /* mask out offset */
   address&=0xFFFFF000;

   if((idx=lookup(owner, address))>=0){
      memcpy(buffer, swap_map[idx].page, PAGE_SIZE);
   }

   END_PHYS_ADDR(info,0);

  return (idx<0) ? 0 : 1;
}

int swapper(int request){
   /*kprintf("swapper invoked: swapper not implemented yet!...halting\n");*/
   for(;;);
}
